/*
 *   snortconf - configuration tool for snort.conf
 *
 *   Copyright (C) 2001 ManISec, Inc.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Yee ole includes (I put this all in one file for my sanity) */
#include "include.h"


/* function to write out the config file based on user choices made in config process     */
/* This should really be broken up into a set of smaller functions instead of one big one */
int snc_write_config() {

	FILE *of;
	unsigned long int i;

	/* open yee ole file	*/	
	of = fopen(SNORT_CONFIG_FILE, "w");
	/* if that didn't work, bail out */
	if(of == NULL) {
		return 1;
	}
	
	/* okie, we got output file, start spitting stuff into it	*/
	fprintf(of, "# This file was generated by SnortConf version %s - Copyright 2002 ManIsec, Inc.\n", VERSION);
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	/* we'll write these config options later in the menu system  */
	
	fprintf(of, "# General Configuration Options\n");
	if(cfg.interface[0] != '\0') fprintf(of, "config interface: %s\n", cfg.interface); 
	if(cfg.logdir[0] != '\0') fprintf(of, "config logdir: %s\n", cfg.logdir);
	if(cfg.set_uid[0] != '\0') fprintf(of, "config set_uid: %s\n", cfg.set_uid);
	if(cfg.set_gid[0] != '\0') fprintf(of, "config set_gid: %s\n", cfg.set_gid);
	  if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == FALSE) fprintf(of, "config ghetto_msg: basic\n");
	  if(cfg.ghetto_msg == TRUE && cfg.ghetto_msg_type == TRUE) fprintf(of, "config ghetto_msg: url\n");
	if(cfg.quiet == TRUE) 		fprintf(of, "config quiet\n");
	if(cfg.dump_payload == TRUE)	fprintf(of, "config dump_payload\n");
	if(cfg.stateful == TRUE)	fprintf(of, "config stateful\n");	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Network Information Configuration\n");
	if(var.home_net[0] == '\0') {		/* we HOPE we never get here	*/
		fprintf(of, "var HOME_NET 192.168.0.0/16\n");
		fprintf(of, "# NOTE: $INTERNAL and $EXTERNAL defined for vision.rules\n");
		fprintf(of, "# They are not required for stock snort *.rules\n");
		fprintf(of, "var INTERNAL $HOME_NET\n");
		fprintf(of, "var EXTERNAL !$HOME_NET\n");
		fprintf(of, "var EXTERNAL_NET !$HOME_NET\n");
		fprintf(of, "var SMTP $HOME_NET\n");
		fprintf(of, "var HTTP_SERVERS $HOME_NET\n");
		fprintf(of, "var SQL_SERVERS $HOME_NET\n");
		fprintf(of, "var DNS_SERVERS $HOME_NET\n");
	} else {	/* this is what we really want		*/
		fprintf(of, "var HOME_NET %s\n", var.home_net);
		fprintf(of, "# NOTE $INTERNAL and $EXTERNAL defined for vision.rules\n");
		fprintf(of, "# They are not required for stock snort *.rules\n");
		fprintf(of, "var INTERNAL %s\n", var.home_net);
		fprintf(of, "var EXTERNAL %s\n", var.external_net);
		fprintf(of, "var EXTERNAL_NET %s\n", var.external_net);
		fprintf(of, "var SMTP %s\n", var.smtp_servers);
		fprintf(of, "var HTTP_SERVERS %s\n", var.http_servers);
		fprintf(of, "var SQL_SERVERS %s\n", var.sql_servers);
		fprintf(of, "var DNS_SERVERS %s\n", var.dns_servers);
	}
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Preprocessor Configuration\n");
	if(plug.frag2 == TRUE) fprintf(of, "preprocessor frag2\n");
	if(plug.stream4 == TRUE) fprintf(of, "preprocessor stream4: detect_scans\n");
	if(plug.stream4_reass == TRUE) fprintf(of, "preprocessor stream4_reassemble\n");
			
	if(plug.http_decode == TRUE) fprintf(of, "preprocessor http_decode: 80 -unicode -cginull\n");
	if(plug.unidecode == TRUE) fprintf(of, "preprocessor unidecode: 80 -unicode -cginull\n");
	if(plug.rpc_decode == TRUE) fprintf(of, "preprocessor rpc_decode: 111 32771\n");
	if(plug.bo_detect == TRUE) fprintf(of, "preprocessor bo: -nobrute 31337\n");
	if(plug.telnet_decode == TRUE) fprintf(of, "preprocessor telnet_decode\n");

	if(plug.spade == TRUE) {
		fprintf(of, "#\n");
		/* if user specified logdir, use that	*/
		if(cfg.logdir[0] != '\0') fprintf(of, "preprocessor spade: 10.5 %s/spade.rcv %s/spade-log.txt 3 50000\n", cfg.logdir, cfg.logdir);
		/* otherwise, assume /var/log/snort/  (bad thing, I know, will fix later */
		else fprintf(of, "preprocessor spade: 10.5 /var/log/snort/spade.rcv /var/log/snort/spade-log.txt 3 50000\n");
		fprintf(of, "preprocessor spade-homenet: $HOME_NET\n");
		fprintf(of, "preprocessor spade-adapt2: 0.01 15 4 24 7\n");
		fprintf(of, "preprocessor spade-threshlearn: 200 24\n");
		/* as above 	*/
		if(cfg.logdir[0] != '\0') fprintf(of, "preprocessor spade-survey:  %s/snort/spade-survey.txt 60\n", cfg.logdir);
		else fprintf(of, "preprocessor spade-survey:  %s/snort/spade-survey.txt 60\n", cfg.logdir);
		fprintf(of, "preprocessor spade-stats: entropy uncondprob condprob\n");
		fprintf(of, "#\n");
	}
	if(plug.portscan == TRUE) {
		fprintf(of, "preprocessor portscan: $HOME_NET 4 3 portscan.log\n");
		fprintf(of, "preprocessor portscan-ignorehosts: %s\n", plug.portscan_ignorehosts);
	}
		
	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Output Plugin Configuration\n");
	if(output.syslog == TRUE) {
		fprintf(of, "output alert_syslog: LOG_AUTH LOG_ALERT LOG_NDELAY\n");
	}

	if(output.tcpdump == TRUE) {
		fprintf(of, "output log_tcpdump: snort.log\n");
	}

	if(output.database == TRUE) {
		switch(output.db_type) {
			case MYSQL:
				fprintf(of, "output database: log, mysql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case PGSQL:
				fprintf(of, "output database: log, postgresql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case MSSQL:
				fprintf(of, "output database: log, mssql, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
			case ODBC:
				fprintf(of, "output database: log, odbc, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
			case ORACLE:
				fprintf(of, "output database: log, oracle, user=%s password=%s dbname=%s host=%s", output.db_user, output.db_pass, output.db_name, output.db_host);
				break;
		}
		if(output.db_port[0] != '\0') {
			fprintf(of, " port=%s\n", output.db_port);
		} else {
			fprintf(of, "\n");
		}
	}
	
	if(output.xml == TRUE) {
		fprintf(of, "output xml: log, file=%s/snortxml\n", cfg.logdir);
	}

	if(output.unified == TRUE) {
		fprintf(of, "output alert_unified: filename snort.alert, limit 128\n");
		fprintf(of, "output log_unified: filename snort.log, limit 128\n");
	}

	if(output.snmp == TRUE) {
		/* too complicated to mess with for now */
	}	
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Include classification.confg and any other default includes\n");
	fprintf(of, "include %s/classification.config\n", SNORT_CONFIG_DIR);
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "#\n");
	fprintf(of, "# Ruleset Selections - you can comment these out as appropo\n");

	
	for(i = rules_files.gl_offs; i < rules_files.gl_pathc ; ++i) {
		if(rules_idx[i] == TRUE) {
			fprintf(of, "include %s\n", rules_files.gl_pathv[i]);
		} else {
			fprintf(of, "# include %s\n", rules_files.gl_pathv[i]);
		}
	}
	fclose(of);	

	return 0;
	
}


	
